/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.event;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.util.AbstractEvent;
import edu.stanford.smi.protege.widget.ClsWidget;

/**
 * Event generated when the project changes.
 *
 * @author    Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class ProjectEvent extends AbstractEvent {
    public static final int FORM_CHANGED = 1;
    public static final int PROJECT_SAVED = 2;
    public static final int PROJECT_CLOSED = 3;
    public static final int RUNTIME_CLS_WIDGET_CREATED = 4;
    public static final int SERVER_SESSION_LOST = 5;

    public ProjectEvent(Project project, int type, ClsWidget widget) {
        super(project, type, widget);
    }

    public ProjectEvent(Project project, int type) {
        super(project, type);
    }

    public Cls getCls() {
        return getClsWidget().getCls();
    }

    public ClsWidget getClsWidget() {
        return (ClsWidget) getArgument();
    }
}
