/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.job;

import edu.stanford.smi.protege.exception.ProtegeException;
import edu.stanford.smi.protege.server.RemoteServer;
import edu.stanford.smi.protege.server.RemoteSession;
import edu.stanford.smi.protege.server.Server;
import edu.stanford.smi.protege.server.ServerProject;
import edu.stanford.smi.protege.server.framestore.ServerFrameStore;
import edu.stanford.smi.protege.util.ServerJob;

public class GetFrameCalculatorStatisticsJob extends ServerJob {
	   
	private static final long serialVersionUID = 1218145961291562140L;

	private String projectName;
	
	public GetFrameCalculatorStatisticsJob(RemoteServer server, RemoteSession session, String projectName) {
        super(server, session);
        this.projectName = projectName;
    }

    @Override
    public Object run() throws ProtegeException {
        Server server = Server.getInstance();
        synchronized (server) {
            ServerProject serverProject = server.getServerProject(projectName);
            if (serverProject == null) {
            	return null;
            }
            ServerFrameStore serverFs = (ServerFrameStore) serverProject.getDomainKbFrameStore(getSession());
            return serverFs.getFrameCalculator().getStats();
		}
    }

}
