/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.metaproject.impl;

import java.io.Serializable;
import java.util.Collection;
import java.util.Set;

import edu.stanford.smi.protege.exception.OntologyException;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.server.metaproject.MetaProjectConstants;
import edu.stanford.smi.protege.server.metaproject.Operation;
import edu.stanford.smi.protege.server.metaproject.PropertyValue;

public class OperationImpl extends WrappedProtegeInstanceImpl implements Operation, Serializable {
	private static final long serialVersionUID = 3175714463454087306L;

	private String name;

	protected OperationImpl(MetaProjectImpl mp, Instance op) {
		super(mp, op, MetaProjectImpl.ClsEnum.Operation);
		name = (String) op.getOwnSlotValue(mp.getSlot(MetaProjectImpl.SlotEnum.name));
	}

	public String getName() {
		return name;
	}

	@Override
    public boolean equals(Object o) {
		if (!(o instanceof Operation))  {
			return false;
		}
		Operation other = (Operation) o;
		return name.equals(other.getName());
	}

	@Override
    public int hashCode() {
		return name.hashCode();
	}

	@Override
    public String toString() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
		setSlotValue(MetaProjectImpl.SlotEnum.name, name);
	}
	
	public String getDescription() {
		Object value = getProtegeInstance().getOwnSlotValue(getMetaProject().getSlot(MetaProjectImpl.SlotEnum.description));
		if (value != null && !(value instanceof String)) {
			throw new OntologyException("The " + MetaProjectImpl.SlotEnum.name + " slot should take on string values");
		}
		return (String) value;		
	}
	
	public void setDescription(String description) {
		setSlotValue(MetaProjectImpl.SlotEnum.description, description);
	}
	
	@SuppressWarnings("unchecked")
	public Set<PropertyValue> getPropertyValues() {
		return getSlotValues(MetaProjectImpl.SlotEnum.properties, MetaProjectImpl.ClsEnum.PropertyValue);
	}

	public void setPropertyValues(Collection<PropertyValue> propertyValues) {
		setSlotValuesAsProtegeInstances(MetaProjectImpl.SlotEnum.properties, propertyValues);		
	}
	
	 public void addPropertyValue(PropertyValue propertyValue) {
    	 addSlotValue(MetaProjectImpl.SlotEnum.properties, propertyValue);		
	}
    
	
	// *********************** Deprecated constants *********************** //
	
	/**
	 * @deprecated - Use constants from {@link MetaProjectConstants}
	 */
	@Deprecated
	public final static Operation READ = MetaProjectConstants.OPERATION_READ;
	
	/**
	 * @deprecated - Use constants from {@link MetaProjectConstants}
	 */
	@Deprecated 
	public final static Operation WRITE = MetaProjectConstants.OPERATION_WRITE;

	/**
	 * @deprecated - Use constants from {@link MetaProjectConstants}
	 */
	@Deprecated
	public final static Operation PROPERTY_TAB_READ = MetaProjectConstants.OPERATION_PROPERTY_TAB_READ;
	
	/**
	 * @deprecated - Use constants from {@link MetaProjectConstants}
	 */
	@Deprecated
	public final static Operation PROPERTY_TAB_WRITE = MetaProjectConstants.OPERATION_ONTOLOGY_TAB_WRITE;
	
	/**
	 * @deprecated - Use constants from {@link MetaProjectConstants}
	 */
	@Deprecated
	public final static Operation ONTOLOGY_TAB_READ = MetaProjectConstants.OPERATION_ONTOLOGY_TAB_READ;
	
	/**
	 * @deprecated - Use constants from {@link MetaProjectConstants}
	 */
	@Deprecated
	public final static Operation ONTOLOGY_TAB_WRITE = MetaProjectConstants.OPERATION_ONTOLOGY_TAB_WRITE;

	
}
