/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.socket;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;


public class TeeAspect implements SocketAspect {
    private SocketAspect delegate;
    private OutputStream os;

    public TeeAspect(SocketAspect delegate) {
        super();
        this.delegate = delegate;
    }
    
    public SocketAspect getDelegate() {
        return delegate;
    }


    public void close() throws IOException {
        if (os != null) {
            os.flush();
            os.close();
        }
        if (getInputStream() != null) {
            getInputStream().close();
        }
    }

    public InputStream getInputStream() throws IOException {
        return delegate.getInputStream();
    }

    public OutputStream getOutputStream() throws IOException {
        if (os == null) {
            os = new TeeOutputStream(delegate.getOutputStream());
        }
        return os;
    }

    public static boolean doTeeOuput() {
        return System.getProperty(TeeOutputStream.TEE_OUTPUT_OPTION) != null;
    }

}
