/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.util;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This is the writer part of a pair of linked classes, the FifoWriter
 * and the FifoReader.
 * 
 * These classes implement a Fifo queue as one writer and many readers.
 * 
 * @author tredmond
 *
 * @param <X>
 */

public class FifoWriter<X> {
  private Logger log;
  private String name;
  /**
   * The Fifo Writer holds the very tail of a linked list
   * which contains a null element and the next item on this
   * queue is null.
   */
  private LinkedList<X> queue = new LinkedList<X>();
  
  public synchronized void write(X x) {
    queue.setElement(x);
    queue.setNext(new LinkedList<X>());
    queue = queue.next();
    if (log != null && log.isLoggable(Level.FINE)) {
      log.fine("Adding " + name + " update " + x);
    }
  }
  
  protected LinkedList<X> getQueue() {
    return queue;
  }
  
  public int getCounter() {
    return queue.getCounter();
  }
  
  public void setLogger(Logger log, String name) {
    this.log = log;
    this.name = name;
  }
}
