/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.util;

import java.awt.event.*;

import javax.swing.*;

import edu.stanford.smi.protege.resource.*;

/**
 * Base class support for the "Add" action available on most widgets.  The semantics of "add" for cardinality single are
 * actually "set" (remove and add).
 *
 * @author    Ray Fergerson <fergerson@smi.stanford.edu>
 */
public abstract class AddAction extends AllowableAction {

    protected AddAction(ResourceKey key) {
        super(key);
    }
    
    protected AddAction(String text) {
        super(text, null);
        if (getIcon() == null) {
            setIcon(Icons.getAddIcon());
        }
    }

    protected AddAction(String text, Icon icon) {
        super("Add", text, icon, null);
    }

    public void actionPerformed(ActionEvent event) {
        if (isAllowed()) {
            onAdd();
        }
    }

    public abstract void onAdd();
}
