/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.widget;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 * Slot widget for acquired a unique value for this own slot in the knowledge base.
 * 
 * @author    Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class UniqueStringWidget extends TextFieldWidget {

    protected String getInvalidTextDescription(String text) {
        String invalidText = null;
        if (text == null || !isUnique(text)) {
            invalidText = "String is not unique";

        }
        return invalidText;
    }

    protected boolean isUnique(String text) {
        Collection frames = getKnowledgeBase().getFramesWithValue(getSlot(), null, false, text);
        boolean isUnique;
        switch (frames.size()) {
            case 0 :
                isUnique = true;
                break;
            case 1 :
                Frame frame = (Frame) CollectionUtilities.getFirstItem(frames);
                isUnique = frame.equals(getInstance());
                break;
            default :
                isUnique = false;
                break;
        }
        return isUnique;
    }
}
